//+------------------------------------------------------------------+
//|                                                          wte.mqh |
//|                             Copyright 2017, Besarion Turmanauli. |
//|                       https://www.mql5.com/en/users/tobeone/blog |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, Besarion Turmanauli."
#property link      "https://www.mql5.com/en/users/tobeone/blog"
#property version   "1.00"
#property strict
//+------------------------------------------------------------------+
//|      double-u (w) trade engine                                   |
//+------------------------------------------------------------------+
class wte
  {
private:

public:
   double            getVolume(double riskPercent,double stopPoints,string instr=NULL);
   void              trailPositions(int magicNumber,double trailPoints,double stepPoints);
   void              trailPositions(int magicNumber,double trailPoints,double stepPoints,int &buyOrders,int &sellOrders);
   void              trailEvenPositions(int magicNumber,double beven,double belock,double trailPoints,double stepPoints,int &buyOrders,int &sellOrders);
   void              openTrade(int orderType,double openPrice,double orderLots,double stopPoints,double limitPoints,string orderComment,int orderMagic,datetime orderExpiration);
   void              managePositions(int magicNumber);
                     wte();
                    ~wte();
  };
//+------------------------------------------------------------------+
//|  get trading vollume (lots)                                      |
//+------------------------------------------------------------------+
double wte::getVolume(double riskPercent,double stopPoints,string instr=NULL)
  {
   RefreshRates();

   double vol=0.01;

   double usableFunds=MathMin(AccountBalance(),AccountEquity());
   double tickValue=MarketInfo(instr,MODE_TICKVALUE);if(tickValue==0)tickValue=1;//get and protect tickValue
   double minLot=MarketInfo(instr,MODE_MINLOT);
   double lotStep= MarketInfo(instr,MODE_LOTSTEP);
   double maxLot = MarketInfo(instr,MODE_MAXLOT);

   vol = NormalizeDouble((usableFunds*(riskPercent/100))/(stopPoints*tickValue),2);
   vol = MathMin(MathMax(minLot,vol),maxLot);
   if(lotStep>0.01)vol=0.01;

   return(vol);
  }
//+------------------------------------------------------------------+
//| trailing stop                                                    |
//+------------------------------------------------------------------+
void wte::trailPositions(int magicNumber,double trailPoints,double stepPoints)
  {
//manage trades here   
   if(OrdersTotal()>0 && trailPoints>0)
     {
      for(int i=0; i<OrdersTotal(); i++)
        {
         if(OrderSelect(i,SELECT_BY_POS))
           {
            if(OrderMagicNumber()==magicNumber && OrderSymbol()==Symbol())
              {
               if(OrderType()==OP_BUY)
                 {
                  //if buy
                  if(
                     MarketInfo(Symbol(),MODE_BID)-OrderOpenPrice()>trailPoints*MarketInfo(Symbol(),MODE_POINT) && //order is in profit
                     MarketInfo(Symbol(),MODE_BID)-OrderStopLoss()>(trailPoints+stepPoints)*MarketInfo(Symbol(),MODE_POINT) //ts level reached
                     )
                    {
                     if(OrderModify(OrderTicket(),OrderOpenPrice(),(MarketInfo(Symbol(),MODE_BID)-trailPoints*MarketInfo(Symbol(),MODE_POINT)),OrderTakeProfit(),OrderExpiration(),clrBlue))
                       {
                        GetLastError();
                       }
                    }

                    }else if(OrderType()==OP_SELL){
                  //if sell
                  if(
                     OrderOpenPrice()-MarketInfo(Symbol(),MODE_ASK)>trailPoints*MarketInfo(Symbol(),MODE_POINT) && //order is in profit
                     OrderStopLoss()-MarketInfo(Symbol(),MODE_ASK)>(trailPoints+stepPoints)*MarketInfo(Symbol(),MODE_POINT) //ts level reached
                     )
                    {
                     if(OrderModify(OrderTicket(),OrderOpenPrice(),(MarketInfo(Symbol(),MODE_ASK)+trailPoints*MarketInfo(Symbol(),MODE_POINT)),OrderTakeProfit(),OrderExpiration(),clrRed))
                       {
                        GetLastError();
                       }
                    }
                 }
              }//end checking if magic number is same 
           }
        }
     }
//end managing trades here...   

  }//end trailing stop
//+------------------------------------------------------------------+
//| trailing stop with counter                                       |
//+------------------------------------------------------------------+
void wte::trailPositions(int magicNumber,double trailPoints,double stepPoints,int &buyOrders,int &sellOrders)
  {
//manage trades here

   if(OrdersTotal()>0)
     {
      for(int i=0; i<OrdersTotal(); i++)
        {
         if(OrderSelect(i,SELECT_BY_POS))
           {
            if(OrderMagicNumber()==magicNumber && OrderSymbol()==Symbol())
              {

               //--- count only
               if(OrderType()==OP_BUY || OrderType()==OP_BUYLIMIT || OrderType()==OP_BUYSTOP)buyOrders++;
               if(OrderType()==OP_SELL || OrderType()==OP_SELLLIMIT || OrderType()==OP_SELLSTOP)sellOrders++;
               //--- end counting only

               if(trailPoints>0)
                 {
                  if(OrderType()==OP_BUY)
                    {
                     //if buy
                     if(
                        MarketInfo(Symbol(),MODE_BID)-OrderOpenPrice()>trailPoints*MarketInfo(Symbol(),MODE_POINT) && //order is in profit
                        MarketInfo(Symbol(),MODE_BID)-OrderStopLoss()>(trailPoints+stepPoints)*MarketInfo(Symbol(),MODE_POINT) //ts level reached
                        )
                       {
                        if(OrderModify(OrderTicket(),OrderOpenPrice(),(MarketInfo(Symbol(),MODE_BID)-trailPoints*MarketInfo(Symbol(),MODE_POINT)),OrderTakeProfit(),OrderExpiration(),clrBlue))
                          {
                           GetLastError();
                          }
                       }

                       }else if(OrderType()==OP_SELL){
                     //if sell
                     if(
                        OrderOpenPrice()-MarketInfo(Symbol(),MODE_ASK)>trailPoints*MarketInfo(Symbol(),MODE_POINT) && //order is in profit
                        OrderStopLoss()-MarketInfo(Symbol(),MODE_ASK)>(trailPoints+stepPoints)*MarketInfo(Symbol(),MODE_POINT) //ts level reached
                        )
                       {
                        if(OrderModify(OrderTicket(),OrderOpenPrice(),(MarketInfo(Symbol(),MODE_ASK)+trailPoints*MarketInfo(Symbol(),MODE_POINT)),OrderTakeProfit(),OrderExpiration(),clrRed))
                          {
                           GetLastError();
                          }
                       }
                    }

                 }//end if trail  

              }//end checking if magic number is same 
           }
        }
     }
//end managing trades here...   

  }//end trailing stop with a double counter
//+------------------------------------------------------------------+
//| trailing stop with counter and breakeven                         |
//+------------------------------------------------------------------+
void wte::trailEvenPositions(int magicNumber,double beven,double belock,double trailPoints,double stepPoints,int &buyOrders,int &sellOrders)
  {
//manage trades here
 
   if(OrdersTotal()>0)
     {
 
      for(int i=0; i<OrdersTotal(); i++)
        {
         if(OrderSelect(i,SELECT_BY_POS))
           {
            if(OrderMagicNumber()==magicNumber && OrderSymbol()==Symbol())
              {
         
               //--- count only
               if(OrderType()==OP_BUY || OrderType()==OP_BUYLIMIT || OrderType()==OP_BUYSTOP)buyOrders++;
               if(OrderType()==OP_SELL || OrderType()==OP_SELLLIMIT || OrderType()==OP_SELLSTOP)sellOrders++;
               //--- end counting only

               bool beApplied=false;
               if(beven>0 && belock>0 && beven>belock)
                 {
                  if(OrderType()==OP_BUY)
                    {
                     //if buy
                     if(
                        MarketInfo(Symbol(),MODE_BID)-OrderOpenPrice()>beven*MarketInfo(Symbol(),MODE_POINT) && //by be points
                        OrderStopLoss()<OrderOpenPrice() //be or ts not applied yet
                        )
                       {
                        if(OrderModify(OrderTicket(),OrderOpenPrice(),(OrderOpenPrice()+belock*MarketInfo(Symbol(),MODE_POINT)),OrderTakeProfit(),OrderExpiration(),clrBlue))
                          {
                           beApplied=true;
                          }
                       }

                       }else if(OrderType()==OP_SELL){
                     //if sell
                     if(
                        OrderOpenPrice()-MarketInfo(Symbol(),MODE_ASK)>beven*MarketInfo(Symbol(),MODE_POINT) && //order is in profit by be pips
                        OrderStopLoss()>OrderOpenPrice() //be or ts not applied yet
                        )
                       {
                        if(OrderModify(OrderTicket(),OrderOpenPrice(),(OrderOpenPrice()-belock*MarketInfo(Symbol(),MODE_POINT)),OrderTakeProfit(),OrderExpiration(),clrRed))
                          {
                           beApplied=true;
                          }
                       }
                    }
                 }

               if(trailPoints>0 && !beApplied)
                 {
                  if(OrderType()==OP_BUY)
                    {
                     //if buy
                     if(
                        MarketInfo(Symbol(),MODE_BID)-OrderOpenPrice()>trailPoints*MarketInfo(Symbol(),MODE_POINT) && //order is in profit
                        MarketInfo(Symbol(),MODE_BID)-OrderStopLoss()>(trailPoints+stepPoints)*MarketInfo(Symbol(),MODE_POINT) //ts level reached
                        )
                       {
                        if(OrderModify(OrderTicket(),OrderOpenPrice(),(MarketInfo(Symbol(),MODE_BID)-trailPoints*MarketInfo(Symbol(),MODE_POINT)),OrderTakeProfit(),OrderExpiration(),clrBlue))
                          {
                           GetLastError();
                          }
                       }

                       }else if(OrderType()==OP_SELL){
                     //if sell
                     if(
                        OrderOpenPrice()-MarketInfo(Symbol(),MODE_ASK)>trailPoints*MarketInfo(Symbol(),MODE_POINT) && //order is in profit
                        OrderStopLoss()-MarketInfo(Symbol(),MODE_ASK)>(trailPoints+stepPoints)*MarketInfo(Symbol(),MODE_POINT) //ts level reached
                        )
                       {
                        if(OrderModify(OrderTicket(),OrderOpenPrice(),(MarketInfo(Symbol(),MODE_ASK)+trailPoints*MarketInfo(Symbol(),MODE_POINT)),OrderTakeProfit(),OrderExpiration(),clrRed))
                          {
                           GetLastError();
                          }
                       }
                    }

                 }//end if trail  

              }//end checking if magic number is same 
           }
        }
     }
//end managing trades here...   

  }//end trailing stop with a double counter and breakeven
//+------------------------------------------------------------------+
//|  OPENTRADE FUNCTION                                              |
//+------------------------------------------------------------------+
void wte::openTrade(int orderType,double openPrice,double orderLots,double stopPoints,double limitPoints,string orderComment,int orderMagic,datetime orderExpiration)
  {

   if(orderType==OP_BUY)
     {//buy order 
      double sl=openPrice-stopPoints*MarketInfo(Symbol(),MODE_POINT);
      double tp=openPrice+limitPoints*MarketInfo(Symbol(),MODE_POINT);

      int ticket=OrderSend(Symbol(),OP_BUY,orderLots,openPrice,3,0,0,orderComment,orderMagic,0,clrBlue);
      if(ticket>0)
        {
         if(OrderSelect(ticket,SELECT_BY_TICKET))
           {
            if(!OrderModify(OrderTicket(),OrderOpenPrice(),sl,tp,OrderExpiration(),clrBlue))
              {
               Print("Couldn't modify the order #"+(string)OrderTicket());
               Print(GetLastError());
               if(!OrderClose(OrderTicket(),OrderLots(),MarketInfo(Symbol(),MODE_BID),3,clrBlue))GetLastError();

              }
           }
           }else{
         Print("BUY Open price: "+(string)openPrice+" &sl: "+(string)sl+" &tp: "+(string)tp);
         Print(GetLastError());
        }
     }//end if buy

   if(orderType==OP_SELL)
     {//selltop order 
      double sl=openPrice+stopPoints*MarketInfo(Symbol(),MODE_POINT);
      double tp=openPrice-limitPoints*MarketInfo(Symbol(),MODE_POINT);

      int ticket=OrderSend(Symbol(),OP_SELL,orderLots,openPrice,3,0,0,orderComment,orderMagic,orderExpiration,clrRed);
      if(ticket>0)
        {
         if(OrderSelect(ticket,SELECT_BY_TICKET))
           {
            if(!OrderModify(OrderTicket(),OrderOpenPrice(),sl,tp,OrderExpiration(),clrRed))
              {
               Print("Couldn't modify the order #"+(string)OrderTicket());
               Print(GetLastError());
               if(!OrderClose(OrderTicket(),OrderLots(),MarketInfo(Symbol(),MODE_ASK),3,clrRed))GetLastError();
              }
           }
           }else{
         Print("SELL Open price: "+(string)openPrice+" &sl: "+(string)sl+" &tp: "+(string)tp);
         Print(GetLastError());
        }
     }//end if sellstop







   if(orderType==OP_BUYSTOP)
     {//buystop order 
      double sl=openPrice-stopPoints*MarketInfo(Symbol(),MODE_POINT);
      double tp=openPrice+limitPoints*MarketInfo(Symbol(),MODE_POINT);

      int ticket=OrderSend(Symbol(),OP_BUYSTOP,orderLots,openPrice,3,0,0,orderComment,orderMagic,orderExpiration,clrBlue);
      if(ticket>0)
        {
         if(OrderSelect(ticket,SELECT_BY_TICKET))
           {
            if(!OrderModify(OrderTicket(),OrderOpenPrice(),sl,tp,OrderExpiration(),clrBlue))
              {
               Print("Couldn't modify the order #"+(string)OrderTicket());
               Print(GetLastError());
               if(!OrderDelete(OrderTicket()))GetLastError();
              }
           }
           }else{
         Print("BUYSTOP Open price: "+(string)openPrice+" &sl: "+(string)sl+" &tp: "+(string)tp);
         Print(GetLastError());
        }
     }//end if buystop

   if(orderType==OP_SELLSTOP)
     {//selltop order 
      double sl=openPrice+stopPoints*MarketInfo(Symbol(),MODE_POINT);
      double tp=openPrice-limitPoints*MarketInfo(Symbol(),MODE_POINT);

      int ticket=OrderSend(Symbol(),OP_SELLSTOP,orderLots,openPrice,3,0,0,orderComment,orderMagic,orderExpiration,clrRed);
      if(ticket>0)
        {
         if(OrderSelect(ticket,SELECT_BY_TICKET))
           {
            if(!OrderModify(OrderTicket(),OrderOpenPrice(),sl,tp,OrderExpiration(),clrRed))
              {
               Print("Couldn't modify the order #"+(string)OrderTicket());
               Print(GetLastError());
               if(!OrderDelete(OrderTicket()))GetLastError();
              }
           }
           }else{
         Print("SELLSTOP Open price: "+(string)openPrice+" &sl: "+(string)sl+" &tp: "+(string)tp);
         Print(GetLastError());
        }
     }//end if sellstop

   if(orderType==OP_BUYLIMIT)
     {//buylimit order 
      double sl=openPrice-stopPoints*MarketInfo(Symbol(),MODE_POINT);
      double tp=openPrice+limitPoints*MarketInfo(Symbol(),MODE_POINT);

      int ticket=OrderSend(Symbol(),OP_BUYLIMIT,orderLots,openPrice,3,0,0,orderComment,orderMagic,orderExpiration,clrBlue);
      if(ticket>0)
        {
         if(OrderSelect(ticket,SELECT_BY_TICKET))
           {
            if(!OrderModify(OrderTicket(),OrderOpenPrice(),sl,tp,OrderExpiration(),clrBlue))
              {
               Print("Couldn't modify the order #"+(string)OrderTicket());
               Print(GetLastError());
               if(!OrderDelete(OrderTicket()))GetLastError();
              }
           }
           }else{
         Print("BUYLIMIT Open price: "+(string)openPrice+" &sl: "+(string)sl+" &tp: "+(string)tp);
         Print(GetLastError());
        }
     }//end if buylimit

   if(orderType==OP_SELLLIMIT)
     {//selllimit order 
      double sl=openPrice+stopPoints*MarketInfo(Symbol(),MODE_POINT);
      double tp=openPrice-limitPoints*MarketInfo(Symbol(),MODE_POINT);

      int ticket=OrderSend(Symbol(),OP_SELLLIMIT,orderLots,openPrice,3,0,0,orderComment,orderMagic,orderExpiration,clrRed);
      if(ticket>0)
        {
         if(OrderSelect(ticket,SELECT_BY_TICKET))
           {
            if(!OrderModify(OrderTicket(),OrderOpenPrice(),sl,tp,OrderExpiration(),clrRed))
              {
               Print("Couldn't modify the order #"+(string)OrderTicket());
               Print(GetLastError());
               if(!OrderDelete(OrderTicket()))GetLastError();
              }
           }
           }else{
         Print("SELLIMIT Open price: "+(string)openPrice+" &sl: "+(string)sl+" &tp: "+(string)tp);
         Print(GetLastError());
        }
     }//end if selllimit   

  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void wte::managePositions(int magicNumber)
  {
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
wte::wte()
  {
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
wte::~wte()
  {
  }
//+------------------------------------------------------------------+
